import click
import tempfile
import zipfile
import os
from loguru import logger
from ..pib import PIB, PIBException
from ..nrfjprog import NRFJProg


@click.group(name='lte')
@click.option('--nrfjprog-log', is_flag=True, help='Enable NRFJProg log.')
@click.pass_context
def cli(ctx, nrfjprog_log):
    '''LTE Modem SoC commands.'''
    ctx.obj['prog'] = NRFJProg('lte', log=nrfjprog_log)


@cli.command('flash')
@click.argument('file', metavar='FILE', type=click.Path(exists=True))
@click.pass_context
def command_flash(ctx, file):
    '''Flash modem firmware.'''

    def progress(text, ctx={'len': 0}):
        if ctx['len']:
            click.echo('\r' + (' ' * ctx['len']) + '\r', nl=False)
        if not text:
            return
        text = f'  {text}'
        ctx['len'] = len(text)
        click.echo(text, nl=text == 'Successfully completed')

    if file.endswith('.zip'):
        zf = zipfile.ZipFile(file)
        namelist = zf.namelist()
        if len(namelist) == 2:
            if 'modem.zip' not in namelist or 'application.hex' not in namelist:
                raise Exception('Invalid file.')

            with tempfile.TemporaryDirectory() as temp_dir:
                zf.extractall(temp_dir)
                with ctx.obj['prog'] as prog:
                    click.echo(f'Flash: modem.zip')
                    prog.program(os.path.join(temp_dir, 'modem.zip'), progress=progress)
                    progress(None)
                    click.echo(f'Flash: application.hex')
                    prog.program(os.path.join(temp_dir, 'application.hex'), progress=progress)
    else:
        with ctx.obj['prog'] as prog:
            click.echo(f'Flash: {file}')
            prog.program(file, progress=progress)

    progress(None)
    click.echo('Successfully completed')


@cli.command('erase')
@click.pass_context
def command_erase(ctx):
    '''Erase modem firmware.'''
    with ctx.obj['prog'] as prog:
        prog.erase_all()


@ cli.command('reset')
@ click.pass_context
def command_reset(ctx):
    '''Reset modem firmware.'''
    with ctx.obj['prog'] as prog:
        prog.reset()


def main():
    cli()



import re
from hardwario.common.pib import PIB as PIBCommon, PIBException


class PIB(PIBCommon):

    HW_VARIANT_LIST = [
        'CDGLS',
        'CGLS',
        'CGS',
        'CGV',
        'CS',
        'CV',
        'L',
        'BCDGLS',
        'BCGLS',
        'BCGS',
        'BCGV',
        'BCS',
        'BCV',
        'BL',
        'ACDGLS',
        'ACGLS',
        'ACGS',
        'ACGV',
        'ACS',
        'ACV',
        'AL',
    ]

    CLAIM_TOKEN = {
        2: (0x45, '33s')
    }
    BLE_PASSKEY = {
        2: (0x66, '17s')
    }

    def __init__(self, buf=None):
        super().__init__(version=2, buf=buf)

    def _update_family(self):
        self._size = self._default_size + 33 + 17

    def set_hw_variant(self, value):
        if value not in self.HW_VARIANT_LIST:
            raise PIBException('Bad Hardware variant not from list.')
        super().set_hw_variant(value)

    def get_claim_token(self):
        return self._unpack(self.CLAIM_TOKEN)

    def set_claim_token(self, value):
        if not re.match(r'^[\dabcdef]{32}$', value) and value != '':
            raise PIBException('Bad Claim token (32 hexadecimal characters).')
        self._pack(self.CLAIM_TOKEN, value)

    def get_ble_passkey(self):
        return self._unpack(self.BLE_PASSKEY)

    def set_ble_passkey(self, value):
        if not re.match(r'^[a-zA-Z0-9]{0,16}$', value):
            raise PIBException('Bad BLE passkey (max 16 characters).')

        self._pack(self.BLE_PASSKEY, value)

    def get_dict(self):
        payload = super().get_dict()
        payload['claim_token'] = self.get_claim_token()
        payload['ble_passkey'] = self.get_ble_passkey()
        return payload



from loguru import logger
from pynrfjprog import HighLevel, APIError, LowLevel
from pynrfjprog.Parameters import *
from .pib import PIB

_api = None


class NRFJProgException(Exception):
    pass


class NRFJProgRTTNoChannels(NRFJProgException):
    pass


class NRFJProg(LowLevel.API):
    def __init__(self, mcu, jlin_sn=None, clock_speed=None, log=False, log_suffix=None):
        self.mcu = mcu
        self.jlin_sn = jlin_sn
        self.clock_speed = clock_speed
        self.log = log
        self.log_suffix = log_suffix
        self._rtt_channels = None

    def open(self):
        try:
            super().__init__(LowLevel.DeviceFamily.UNKNOWN, log=self.log)
            super().open()

            if self.jlin_sn is not None:
                self.connect_to_emu_with_snr(self.jlin_sn)
            else:
                self.connect_to_emu_without_snr()

        except APIError.APIError as e:
            if e.err_code == APIError.NrfjprogdllErr.NO_EMULATOR_CONNECTED:
                raise NRFJProgException(
                    'No J-Link found (check USB cable)')
            if e.err_code == APIError.NrfjprogdllErr.LOW_VOLTAGE:
                raise NRFJProgException(
                    'Detected low voltage on J-Link (check power supply and cable)')
            raise NRFJProgException(str(e))

        device_family = self.read_device_family()

        if self.mcu == 'app':
            if device_family != 'NRF52':
                raise NRFJProgException(
                    'Detected bad MCU expect: app (NRF52)')
        elif self.mcu == 'lte':
            if device_family != 'NRF91':
                raise NRFJProgException(
                    'Detected bad MCU expect: lte (NRF91)')
        else:
            raise NRFJProgException(
                f'Unknown MCU family ({device_family}).')

        self.select_family(device_family)

        # print(self.read_device_info())

    def close(self):
        super().close()

    def reset(self):
        self.sys_reset()

    def erase_flash(self):
        self.disable_bprot()
        for des in self.read_memory_descriptors(False):
            if des.type == MemoryType.CODE:
                page_size = des.size // des.num_pages
                for addr in range(0, des.size, page_size):
                    self.erase_page(addr)

    def program(self, file_path, halt=False, progress=lambda x: None):
        self.reset()
        self.halt()

        progress('Erasing...')
        self.erase_file(file_path, chip_erase_mode=EraseAction.ERASE_SECTOR)

        progress('Flashing...')
        self.program_file(file_path)

        progress('Verifying...')
        self.verify_file(file_path)

        if halt:
            progress('Resetting (HALT)...')
            self.reset()
            self.halt()
        else:
            progress('Resetting (GO)...')
            self.reset()
            self.go()

        progress('Successfully completed')

    def get_uicr_address(self):
        for des in self.read_memory_descriptors(False):
            if des.type == MemoryType.UICR:
                return des.start
        raise NRFJProgException('UICR descriptor not found.')

    def write_uicr(self, buffer: bytes, halt=False):
        self.reset()
        self.halt()

        self.erase_uicr()
        self.write(self.get_uicr_address() + 0x80, buffer, True)

        self.reset()
        if halt:
            self.halt()
        else:
            self.go()

    def read_uicr(self):
        return bytes(self.read(self.get_uicr_address() + 0x80, 128))

    def rtt_start(self):
        if self._rtt_channels is not None:
            return self._rtt_channels

        super().rtt_start()
        logger.debug('RTT Start')

        for _ in range(100):
            if self.rtt_is_control_block_found():
                logger.debug('RTT control block found')
                break
            time.sleep(0.1)
        else:
            raise NRFJProgException('Can not found RTT start block')

        channel_count = self.rtt_read_channel_count()
        logger.debug(f'RTT channel count {channel_count}')

        channels = {}
        for index in range(channel_count[0]):
            name, size = self.rtt_read_channel_info(index, RTTChannelDirection.DOWN_DIRECTION)
            if size < 1:
                continue
            channels[name] = {
                'down': {
                    'index': index,
                    'size': size
                }
            }
        for index in range(channel_count[1]):
            name, size = self.rtt_read_channel_info(index, RTTChannelDirection.UP_DIRECTION)
            if size < 1:
                continue
            if name not in channels:
                channels[name] = {}
            channels[name]['up'] = {
                'index': index,
                'size': size
            }

        self._rtt_channels = channels
        return self._rtt_channels

    def rtt_stop(self):
        if self._rtt_channels is None:
            return

        # super().rtt_stop() #  WHY: if call rtt_stop then Can not found RTT start block after rtt_start, needs reset for work
        self._rtt_channels = None
        logger.debug('RTT Stop')

    def rtt_is_running(self):
        return self._rtt_channels is not None

    def rtt_write(self, channel, msg, encoding='utf-8'):
        if self._rtt_channels is None:
            raise NRFJProgRTTNoChannels('Can not write, try call rtt_start first')
        if isinstance(channel, str):
            channel = self._rtt_channels[channel]['down']['index']
        logger.debug('channel: {} msg: {}', channel, repr(msg))
        return super().rtt_write(channel, msg, encoding)

    def rtt_read(self, channel, length=None, encoding='utf-8', ):
        if self._rtt_channels is None:
            raise NRFJProgRTTNoChannels('Can not read, try call rtt_start first')
        if isinstance(channel, str):
            ch = self._rtt_channels[channel]['up']
            if length is None:
                length = ch['size']
            channel = ch['index']
        msg = super().rtt_read(channel, length, encoding)
        if msg:
            logger.debug('channel: {} msg: {}', channel, repr(msg))
        return msg

    def __enter__(self):
        self.open()
        return self

    def __exit__(self, type, value, traceback):
        self.close()


class HighNRFJProg(HighLevel.DebugProbe):

    def __init__(self, mcu, jlin_sn=None, clock_speed=None, log=False, log_suffix=None):
        self.mcu = mcu
        self.jlin_sn = jlin_sn
        self.clock_speed = clock_speed
        self.log = log
        self.log_suffix = log_suffix

    def open(self):
        jlin_sn = self.jlin_sn
        api = get_api()
        if jlin_sn is None:
            probes = api.get_connected_probes()
            if not probes:
                raise NRFJProgException('No J-Link found (check USB cable)')
            jlin_sn = probes[0]

        try:
            super().__init__(api, jlin_sn, clock_speed=self.clock_speed, log=self.log)
        except APIError.APIError as e:
            if e.err_code == APIError.NrfjprogdllErr.LOW_VOLTAGE:
                raise NRFJProgException(
                    'Detected low voltage on J-Link (check power supply and cable)')
            else:
                raise NRFJProgException(str(e))

        self.info = self.get_device_info()
        # print(self.info.__dict__)

        if self.mcu == 'app':
            if self.info.device_family != DeviceFamily.NRF52:
                raise NRFJProgException(
                    'Detected bad MCU expect: app (NRF52)')
        elif self.mcu == 'lte':
            if self.info.device_family != DeviceFamily.NRF91:
                raise NRFJProgException(
                    'Detected bad MCU expect: lte (NRF91)')
        else:
            raise NRFJProgException(
                f'Unknown MCU family ({self.info.device_family})')

    def erase_uicr(self):
        self.erase(EraseAction.ERASE_SECTOR_AND_UICR,
                   self.info.uicr_address, self.info.uicr_address + 32)

    def erase_all(self):
        self.erase(EraseAction.ERASE_ALL)

    def erase_flash(self):
        self.erase(EraseAction.ERASE_SECTOR,
                   self.info.code_address, (self.info.code_size // 32))

    def program(self, hex_path):
        program_options = ProgramOptions(
            verify=VerifyAction.VERIFY_READ,
            erase_action=EraseAction.ERASE_SECTOR,
            qspi_erase_action=EraseAction.ERASE_NONE,
            reset=ResetAction.RESET_SYSTEM
        )
        super().program(hex_path, program_options)

    def write_uicr(self, buffer: bytes):
        self.erase_uicr()
        self.write(self.info.uicr_address + 0x80, buffer)

    def read_uicr(self):
        return self.read(self.info.uicr_address + 0x80, 128)


def get_api():
    global _api
    if _api is None:
        api = HighLevel.API(True)
        api.open()
        _api = api
    return _api


def get_probe(jlin_sn=None):
    api = get_api()
    if not jlin_sn:
        probes = api.get_connected_probes()
        if not probes:
            raise NRFJProgException('No J-Link found (check USB cable)')
        jlin_sn = probes[0]

    try:
        probe = HighLevel.DebugProbe(get_api(), jlin_sn, log=False)
        return probe
    except APIError.APIError as e:
        if e.err_code == APIError.NrfjprogdllErr.LOW_VOLTAGE:
            raise NRFJProgException(
                'Detected low voltage on J-Link (check power supply and cable)')
        else:
            raise NRFJProgException(str(e))


# https://www.segger.com/downloads/jlink/JLink_Linux_x86_64.deb
# https://www.segger.com/downloads/jlink/JLink_Linux_arm.tgz
# https://www.segger.com/downloads/jlink
# {'device_type': < DeviceVersion.NRF52840_xxAA_REV2: 86523907 > , 'device_family': < DeviceFamily.NRF52: 1 > , 'code_address': 0, 'code_page_size': 4096, 'code_size': 1048576, 'uicr_address': 268439552, 'info_page_size': 4096, 'code_ram_present': True, 'code_ram_address': 8388608, 'data_ram_address': 536870912, 'ram_size': 262144, 'qspi_present': True, 'xip_address': 301989888, 'xip_size': 0, 'pin_reset_pin': 18, 'dll_ret_code': 0}



import os
from os.path import join, exists, abspath, isfile, getsize, expanduser
import hashlib
import click
import requests
from loguru import logger

DEFAULT_CACHE_PATH = expanduser("~/.hardwario/chester/cache")


def test_file(*paths):
    file_path = join(*paths)
    if exists(file_path) and isfile(file_path) and getsize(file_path) > 0:
        return file_path


def find_hex(app_path, no_exception=False):
    out_path = join(app_path, 'build', 'zephyr')

    for name in ('merged.hex', 'zephyr.hex'):
        hex_path = test_file(out_path, name)
        if hex_path:
            return hex_path

    if no_exception:
        return None

    raise Exception('No firmware found.')


def download_url(url, filename=None, cache_path=DEFAULT_CACHE_PATH):

    if not filename:
        if url.startswith("https://firmware.hardwario.com/chester"):
            filename = url[39:].replace('/', '.')
        else:
            filename = hashlib.sha256(url.encode()).hexdigest()

    if cache_path:
        os.makedirs(cache_path, exist_ok=True)
        filename = join(cache_path, filename)
        if os.path.exists(filename):
            return filename

    response = requests.get(url, stream=True, allow_redirects=True)
    if response.status_code != 200:
        raise Exception(response.text)
    total_length = response.headers.get('content-length')
    with open(filename, "wb") as f:
        if total_length is None:  # no content length header
            f.write(response.content)
        else:
            with click.progressbar(length=int(total_length), label='Download ') as bar:
                dl = 0
                for data in response.iter_content(chunk_size=4096):
                    dl += len(data)
                    f.write(data)
                    bar.update(dl)
    return filename



import click
from . import app, lte


@click.group(name='chester', help='Commands for CHESTER (configurable IoT gateway).')
@click.pass_context
def cli(ctx):
    pass


cli.add_command(app.cli)
cli.add_command(lte.cli)


def main():
    cli(obj={})



import requests
import subprocess
from loguru import logger
from .utils import find_hex, test_file
from hardwario.common.util import get_file_hash

# DEFAULT_API_URL = 'http://0.0.0.0:4000/chester/api'
DEFAULT_API_URL = 'https://firmware.hardwario.com/chester/api'


class FirmwareApiException(Exception):
    pass


class FirmwareApi:

    def __init__(self, url=DEFAULT_API_URL, token=None):
        self._headers = {}
        self._url = url
        if token:
            self.set_token(token)

    def set_token(self, token):
        self._headers['Authorization'] = 'Bearer ' + token

    def request(self, method, url, **kwargs):
        url = self._url + url
        logger.debug('{} {}', url, kwargs)
        try:
            self._response = requests.request(
                method, url, headers=self._headers, **kwargs)
        except ConnectionError:
            raise FirmwareApiException('Cannot connect to cloud service')

        if 200 < self._response.status_code >= 300:
            text = self._response.text.strip('"')
            raise FirmwareApiException(f'{self._response.status_code}: {text}')

        return self._response.json()

    def upload(self, label, app_path='.'):
        logger.debug(f'label={label}')

        revision = None
        try:
            revision = subprocess.check_output(
                ['git', '-C', app_path, 'rev-parse', 'HEAD']).decode('ascii').strip()
        except Exception:
            pass

        data = {
            'label': label,
            'revision': revision
        }

        files = {}

        hex_path = find_hex(app_path)
        logger.debug(f'hex_path={hex_path}')

        data['firmware_sha256'] = get_file_hash(hex_path)
        files['firmware_hex'] = open(hex_path, 'rb')

        app_update_path = test_file(app_path, 'build', 'zephyr', 'app_update.bin')
        logger.debug(f'app_update_path={app_update_path}')
        if app_update_path:
            data['app_update_sha256'] = get_file_hash(app_update_path)
            files['app_update_bin'] = open(app_update_path, 'rb')

        manifest_json_path = test_file(app_path, 'build', 'zephyr', 'dfu_application.zip_manifest.json')
        logger.debug(f'manifest_json_path={manifest_json_path}')

        if manifest_json_path:
            files['manifest'] = open(manifest_json_path, 'rb')

        resp = self.request('POST', '/v1/firmware', data=data, files=files)
        logger.debug(f'Response {resp}')
        return resp

    def list(self, offset: int = 0, limit: int = None):
        return self._list('/v1/firmware', {}, offset, limit)

    def detail(self, id):
        return self.request('GET', f'/v1/firmware/{id}')

    def delete(self, id):
        return self.request('DELETE', f'/v1/firmware/{id}')

    def _list(self, url, params: dict, offset=0, limit=None):
        cnt = 0
        params['offset'] = offset

        while True:
            params['limit'] = 100 if limit is None else limit

            for row in self.request('GET', url, params=params):
                cnt += 1
                yield row

            if limit is not None and limit >= cnt:
                break

            x_total = int(self._response.headers['x-total'])
            params['offset'] = offset + cnt

            if params['offset'] >= x_total:
                break



import click
import os
import json
import sys
import string
from datetime import datetime
from loguru import logger
from ..pib import PIB, PIBException
from ..nrfjprog import NRFJProg, HighNRFJProg
from ..console import Console
from ..firmwareapi import FirmwareApi, DEFAULT_API_URL
from ..utils import find_hex, download_url


@click.group(name='app')
@click.option('--nrfjprog-log', is_flag=True, help='Enable nrfjprog log.')
@click.pass_context
def cli(ctx, nrfjprog_log):
    '''Application SoC commands.'''
    ctx.obj['prog'] = NRFJProg('app', log=nrfjprog_log)


def validate_hex_file(ctx, param, value):
    # print('validate_hex_file', ctx.obj, param.name, value)
    if len(value) == 32 and all(c in string.hexdigits for c in value):
        return download_url(f'https://firmware.hardwario.com/chester/{value}/hex', filename=f'{value}.hex')

    if os.path.exists(value):
        return value

    raise click.BadParameter(f'Path \'{value}\' does not exist.')


@cli.command('flash')
@click.option('--halt', is_flag=True, help='Halt program.')
@click.argument('hex_file', metavar='HEX_FILE_OR_ID', callback=validate_hex_file, default=find_hex('.', no_exception=True))
@click.pass_context
def command_flash(ctx, halt, hex_file):
    '''Flash application firmware (preserves UICR area).'''
    click.echo(f'File: {hex_file}')

    def progress(text, ctx={'len': 0}):
        if ctx['len']:
            click.echo('\r' + (' ' * ctx['len']) + '\r', nl=False)
        if not text:
            return
        ctx['len'] = len(text)
        click.echo(text, nl=text == 'Successfully completed')

    with ctx.obj['prog'] as prog:
        prog.program(hex_file, halt, progress=progress)


@cli.command('erase')
@click.option('--all', is_flag=True, help='Erase application firmware incl. UICR area.')
@click.pass_context
def command_erase(ctx, all):
    '''Erase application firmware w/o UICR area.'''
    with ctx.obj['prog'] as prog:
        if all:
            prog.erase_all()
        else:
            prog.erase_flash()


@cli.command('reset')
@click.option('--halt', is_flag=True, help='Halt program.')
@click.pass_context
def command_reset(ctx, halt):
    '''Reset application firmware.'''
    with ctx.obj['prog'] as prog:
        prog.reset()
        if halt:
            prog.halt()


default_history_file = os.path.expanduser("~/.chester_history")
default_console_file = os.path.expanduser("~/.chester_console")


@cli.command('console')
@click.option('--reset', is_flag=True, help='Reset application firmware.')
@click.option('--latency', type=int, help='Latency for RTT readout in ms.', show_default=True, default=50)
@click.option('--history-file', type=click.Path(writable=True), show_default=True, default=default_history_file)
@click.option('--console-file', type=click.File('a', 'utf-8'), show_default=True, default=default_console_file)
@click.pass_context
def command_console(ctx, reset, latency, history_file, console_file):
    '''Start interactive console for shell and logging.'''
    logger.remove(2)  # Remove stderr logger

    with ctx.obj['prog'] as prog:
        if reset:
            prog.reset()
            prog.go()
        Console(prog, history_file, console_file, latency=latency)


def validate_pib_param(ctx, param, value):
    # print('validate_pib_param', ctx.obj, param.name, value)
    try:
        getattr(ctx.obj['pib'], f'set_{param.name}')(value)
    except PIBException as e:
        raise click.BadParameter(str(e))
    return value


@cli.group(name='pib')
@click.pass_context
def group_pib(ctx):
    '''HARDWARIO Product Information Block.'''
    ctx.obj['pib'] = PIB()
    pass


@group_pib.command('read')
@click.option('--json', 'out_json', is_flag=True, help='Output in JSON format.')
@click.pass_context
def command_pib_read(ctx, out_json):
    '''Read HARDWARIO Product Information Block from UICR.'''

    with ctx.obj['prog'] as prog:
        buffer = prog.read_uicr()

    pib = PIB(buffer)

    if out_json:
        click.echo(json.dumps(pib.get_dict()))
    else:
        click.echo(f'Vendor name: {pib.get_vendor_name()}')
        click.echo(f'Product name: {pib.get_product_name()}')
        click.echo(f'Hardware variant: {pib.get_hw_variant()}')
        click.echo(f'Hardware revision: {pib.get_hw_revision()}')
        click.echo(f'Serial number: {pib.get_serial_number()}')
        click.echo(f'Claim token: {pib.get_claim_token()}')
        click.echo(f'BLE passkey: {pib.get_ble_passkey()}')


@group_pib.command('write')
@click.option('--vendor-name', type=str, help='Vendor name (max 16 characters).', default='HARDWARIO', prompt=True, show_default=True, callback=validate_pib_param)
@click.option('--product-name', type=str, help='Product name (max 16 characters).', default='CHESTER-M', prompt=True, show_default=True, callback=validate_pib_param)
@click.option('--hw-variant', type=click.Choice(PIB.HW_VARIANT_LIST), help='Hardware variant.', default='', prompt='Hardware variant', show_default=True, callback=validate_pib_param)
@click.option('--hw-revision', type=str, help='Hardware revision in Rx.y format.', default='R3.2', prompt='Hardware revision', show_default=True, callback=validate_pib_param)
@click.option('--serial-number', type=str, help='Serial number in decimal format.', prompt=True, callback=validate_pib_param)
@click.option('--claim-token', type=str, help='Claim token for device self-registration (32 hexadecimal characters).', default='', prompt=True, show_default=True, callback=validate_pib_param)
@click.option('--ble-passkey', type=str, help='Bluetooth security passkey (max 16 characters).', default='123456', prompt=True, show_default=True, callback=validate_pib_param)
@click.option('--halt', is_flag=True, help='Halt program.')
@click.pass_context
def command_pib_write(ctx, vendor_name, product_name, hw_variant, hw_revision, serial_number, claim_token, ble_passkey, halt):
    '''Write HARDWARIO Product Information Block to UICR.'''
    logger.debug('command_pib_write: %s', (serial_number,
                 vendor_name, product_name, hw_revision, hw_variant, claim_token, ble_passkey))

    pib = ctx.obj['pib']
    buffer = pib.get_buffer()

    logger.debug('write uicr: %s', buffer.hex())

    with ctx.obj['prog'] as prog:
        prog.write_uicr(buffer, halt=halt)


@cli.group(name='uicr')
def group_uicr():
    '''UICR flash area.'''
    pass


@group_uicr.command('read')
@click.option('--format', type=click.Choice(['hex', 'bin']), help='Specify input format.', required=True)
@click.argument('file', type=click.File('wb'))
@click.pass_context
def command_uicr_read(ctx, format, file):
    '''Read generic UICR flash area to <FILE> or stdout.'''

    with ctx.obj['prog'] as prog:
        buffer = prog.read_uicr()

    if format == 'hex':
        file.write(buffer.hex().encode())

    elif format == 'bin':
        file.write(buffer)


@group_uicr.command('write')
@click.option('--format', type=click.Choice(['hex', 'bin']), help='Specify input format.', required=True)
@click.option('--halt', is_flag=True, help='Halt program.')
@click.argument('file', type=click.File('rb'))
@click.pass_context
def command_uicr_write(ctx, format, halt, file):
    '''Write generic UICR flash area from <FILE> or stdout.'''

    buffer = file.read()

    if buffer and format == 'hex':
        buffer = bytes.fromhex((''.join(buffer.decode())).strip())

    if buffer is None:
        raise click.BadParameter('Problem load buffer.')

    if len(buffer) > 128:
        raise click.BadParameter('Buffer has wrong size allowed is max 128B')

    logger.debug('write uicr: %s', buffer.hex())

    with ctx.obj['prog'] as prog:
        prog.write_uicr(buffer, halt=halt)


@cli.group(name='fw')
@click.option('--url', metavar='URL', required=True, default=os.environ.get('HARDWARIO_FW_API_URL', DEFAULT_API_URL), show_default=True)
@click.option('--token', metavar='TOKEN', required='--help' not in sys.argv, envvar='HARDWARIO_CLOUD_TOKEN')
@click.pass_context
def cli_fw(ctx, url, token):
    '''Firmware commands.'''
    ctx.obj['fwapi'] = FirmwareApi(url=url, token=token)


@cli_fw.command('upload')
@click.option('--label', type=str, help='Firmware label (max 100 characters).', prompt=True, required=True)
@click.pass_context
def command_fw_upload(ctx, label):
    '''Upload application firmware.'''
    fw = ctx.obj['fwapi'].upload(label, '.')
    click.echo(f'UUID: {fw["id"]}')
    click.echo(f'URL: https://firmware.hardwario.com/chester/{fw["id"]}')


@cli_fw.command('list')
@click.option('--limit', type=click.IntRange(0, 100, clamp=True))
@click.pass_context
def command_fw_upload(ctx, limit):
    '''List application firmwares.'''
    click.echo(f'{"UUID":32} {"Time":19} Label')
    for fw in ctx.obj['fwapi'].list(limit=limit):
        dt = datetime.fromtimestamp(fw['timestamp'])
        click.echo(f'{fw["id"]} {dt} {fw["label"]}')


@cli_fw.command('delete')
@click.option('--id', metavar="ID", required=True)
@click.confirmation_option(prompt='Are you sure you want to delete firmware ?')
@click.pass_context
def command_fw_delete(ctx, id):
    '''Delete firmware.'''
    fw = ctx.obj['fwapi'].delete(id)
    click.echo('OK')


@cli_fw.command('show')
@click.option('--id', metavar="ID", show_default=True, required=True)
@click.pass_context
def command_fw_show(ctx, id):
    '''Show firmware detail.'''
    fw = ctx.obj['fwapi'].detail(id)
    click.echo(f'UUID: {fw["id"]}')
    click.echo(f'URL: https://firmware.hardwario.com/chester/{fw["id"]}')
    click.echo(f'Label: {fw["label"]}')
    click.echo(f'Time: {datetime.fromtimestamp(fw["timestamp"])}')
    click.echo(f'Revision: {fw["revision"]}')
    click.echo(f'SHA256 firmware: {fw["firmware_sha256"]}')
    click.echo(f'SHA256 app_update: {fw["app_update_sha256"]}')
    click.echo(f'Build Manifest: {json.dumps(fw["manifest"])}')


def main():
    cli()



import threading
import os
import asyncio
import logging
import re
from functools import partial
from datetime import datetime
from loguru import logger
from prompt_toolkit.eventloop.utils import get_event_loop
from prompt_toolkit.application import Application
from prompt_toolkit.buffer import Buffer
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.layout.containers import HSplit, VSplit, Window, WindowAlign
from prompt_toolkit.layout.controls import BufferControl, FormattedTextControl
from prompt_toolkit.layout.layout import Layout
from prompt_toolkit.layout.margins import NumberedMargin, ScrollbarMargin
from prompt_toolkit.widgets import SearchToolbar, TextArea, Frame, HorizontalLine
from prompt_toolkit.styles import Style
from prompt_toolkit.key_binding.bindings.focus import focus_next, focus_previous
from prompt_toolkit.document import Document
from prompt_toolkit.history import FileHistory
from prompt_toolkit.clipboard.pyperclip import PyperclipClipboard
from prompt_toolkit.application.current import get_app
from prompt_toolkit.lexers import Lexer
from prompt_toolkit.styles.named_colors import NAMED_COLORS

from .nrfjprog import NRFJProg, NRFJProgRTTNoChannels


def getTime():
    return datetime.now().strftime('%Y.%m.%d %H:%M:%S.%f')[:23]


log_level_color_lut = {
    'X': NAMED_COLORS['Blue'],
    'D': NAMED_COLORS['Magenta'],
    'I': NAMED_COLORS['Green'],
    'W': NAMED_COLORS['Yellow'],
    'E': NAMED_COLORS['Red'],
    'dbg': NAMED_COLORS['Magenta'],
    'inf': NAMED_COLORS['Green'],
    'wrn': NAMED_COLORS['Yellow'],
    'err': NAMED_COLORS['Red'],
}


class LogLexer(Lexer):

    def __init__(self, patern, colors=log_level_color_lut) -> None:
        super().__init__()
        self.patern = patern
        self.colors = colors

    def lex_document(self, document):
        def get_line(lineno):
            line = document.lines[lineno]

            g = re.match(self.patern, line)
            if g:
                color = self.colors.get(g.group(2), '#ffffff')
                return [(color, g.group(1)), ('#ffffff', g.group(3))]

            return [('#ffffff', line)]

        return get_line


class Console:

    def __init__(self, prog: NRFJProg, history_file, console_file, latency=50):
        channels = prog.rtt_start()

        is_old = False

        if 'Terminal' not in channels:
            raise Exception('Not found RTT Terminal channel')

        if len(channels) > 1:
            if 'Logger' not in channels:
                raise Exception('Not found RTT Logger channel')
        elif len(channels) == 1:
            is_old = True

        shell_search = SearchToolbar(ignore_case=True, vi_mode=True)
        shell_window = TextArea(
            scrollbar=True,
            line_numbers=True,
            focusable=True,
            focus_on_click=True,
            read_only=True,
            search_field=shell_search
        )
        self.shell_buffer = shell_window.buffer

        logger_search = SearchToolbar(ignore_case=True, vi_mode=True)
        logger_window = TextArea(
            scrollbar=True,
            line_numbers=True,
            focusable=True,
            focus_on_click=True,
            read_only=True,
            search_field=logger_search,
            lexer=LogLexer(r'^(#.*?\d(?:\.\d+)? <(\w)\>)(.*)' if is_old else r'^(\[.*?\].*?<(\w+)\>)(.*)')
        )
        self.logger_buffer = logger_window.buffer
        logger.debug(f'history_file: {history_file}')

        os.makedirs(os.path.dirname(history_file), exist_ok=True)

        input_history = FileHistory(history_file)
        search_field = SearchToolbar(ignore_case=True)

        self.input_field = TextArea(
            height=1,
            prompt=">>> ",
            style="class:input-field",
            multiline=False,
            wrap_lines=False,
            search_field=search_field,
            history=input_history,
            focusable=True,
            focus_on_click=True)

        def get_titlebar_text():
            return [
                ("class:title", " HARDWARIO CHESTER Console "),
                ("class:title", " (Press [Ctrl-Q] to quit.)"),
            ]

        root_container = HSplit(
            [
                # The titlebar.
                Window(
                    height=1,
                    content=FormattedTextControl(get_titlebar_text),
                    align=WindowAlign.CENTER,
                ),
                VSplit(
                    [
                        Frame(HSplit(
                            [
                                shell_window,
                                shell_search,
                                HorizontalLine(),
                                self.input_field,
                                search_field
                            ]
                        ), title="Shell"),
                        Frame(HSplit(
                            [
                                logger_window,
                                logger_search
                            ]
                        ), title="Log"),
                    ]
                )
            ]
        )

        bindings = KeyBindings()

        @bindings.add("c-insert", eager=True)  # TODO: check
        @bindings.add("c-c", eager=True)
        def do_copy(event):
            if event.app.layout.has_focus(shell_window):
                data = shell_window.buffer.copy_selection()
                event.app.clipboard.set_data(data)

            elif event.app.layout.has_focus(logger_window):
                data = logger_window.buffer.copy_selection()
                event.app.clipboard.set_data(data)

        @bindings.add("c-q", eager=True)
        def _(event):
            event.app.exit()

        bindings.add("tab")(focus_next)
        bindings.add("s-tab")(focus_previous)

        self.app = Application(
            layout=Layout(root_container, focused_element=self.input_field),
            key_bindings=bindings,
            mouse_support=True,
            full_screen=True,
            enable_page_navigation_bindings=True,
            clipboard=PyperclipClipboard()
        )

        rtt_read_delay = latency / 1000.0

        if is_old:
            async def task_rtt_read():
                while prog.rtt_is_running:
                    with logger.catch(message='task_rtt_read', reraise=True):
                        try:
                            lines = prog.rtt_read('Terminal')
                        except NRFJProgRTTNoChannels:
                            return
                        if lines:
                            shell = ''
                            log = ''
                            for line in lines.splitlines():
                                if line.startswith('#'):
                                    log += line + '\n'
                                    console_file.write(getTime() + ' ')
                                    console_file.write(line)
                                else:
                                    shell += line + '\n'
                                    console_file.write(getTime() + ' > ')
                                    console_file.write(line)
                                console_file.write('\n')
                            console_file.flush()

                            if shell:
                                shell = shell.replace('\r', '')
                                self.shell_buffer.set_document(Document(self.shell_buffer.text + shell, None), True)
                            if log:
                                log = log.replace('\r', '')
                                self.logger_buffer.set_document(Document(self.logger_buffer.text + log, None), True)

                        await asyncio.sleep(rtt_read_delay)

        else:
            channels_up = (('Terminal', self.shell_buffer), ('Logger', self.logger_buffer))

            async def task_rtt_read():
                while prog.rtt_is_running:
                    for channel, buffer in channels_up:
                        with logger.catch(message='task_rtt_read', reraise=True):
                            try:
                                line = prog.rtt_read(channel)
                            except NRFJProgRTTNoChannels:
                                return
                            if line:
                                # buffer.insert_text(line.replace('\r', ''))
                                for sline in line.splitlines():
                                    console_file.write(getTime() + (' # ' if channel == 'Logger' else ' > '))
                                    console_file.write(sline)
                                    console_file.write('\n')
                                console_file.flush()

                                line = line.replace('\r', '')
                                buffer.set_document(Document(buffer.text + line, None), True)
                            await asyncio.sleep(rtt_read_delay)

        console_file.write(f'{ "*" * 80 }\n')

        loop = get_event_loop()
        loop.create_task(task_rtt_read())

        def accept(buff):
            line = f'{buff.text}\n'.replace('\r', '')
            # self.shell_buffer.insert_text(line)
            console_file.write(f'{getTime()} < {line}')
            text = self.shell_buffer.text + line
            self.shell_buffer.set_document(Document(text, None), True)

            prog.rtt_write('Terminal', f'{buff.text}\n')
            return None

        self.input_field.accept_handler = accept

        self.app.run()

        prog.rtt_stop()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['hardwario', 'hardwario.chester', 'hardwario.chester.cli']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1',
 'hardwario-common>=1.7.1,<2.0.0',
 'loguru>=0.6.0,<0.7.0',
 'prompt-toolkit>=3.0.28,<4.0.0',
 'pylink-square>=0.12.0,<0.13.0',
 'pynrfjprog==10.16.0',
 'pyperclip>=1.8.2,<2.0.0',
 'requests>=2.28.0,<3.0.0']

entry_points = \
{'console_scripts': ['chester = hardwario.chester.cli:main']}

setup_kwargs = {
    'name': 'hardwario-chester',
    'version': '1.15.1',
    'description': 'HARDWARIO CHESTER',
    'long_description': '<a href="https://www.hardwario.com/"><img src="https://www.hardwario.com/ci/assets/hw-logo.svg" width="200" alt="HARDWARIO Logo" align="right"></a>\n\n# HARDWARIO CHESTER CLI Tools\n\n[![Main](https://github.com/hardwario/py-hardwario-chester/actions/workflows/main.yaml/badge.svg)](https://github.com/hardwario/py-hardwario-chester/actions/workflows/main.yaml)\n[![Release](https://img.shields.io/github/release/hardwario/py-hardwario-chester.svg)](https://github.com/hardwario/py-hardwario-chester/releases)\n[![PyPI](https://img.shields.io/pypi/v/hardwario-chester.svg)](https://pypi.org/project/hardwario-chester/)\n[![License](https://img.shields.io/github/license/hardwario/py-hardwario-chester.svg)](https://github.com/hardwario/py-hardwario-chester/blob/master/LICENSE)\n[![Twitter](https://img.shields.io/twitter/follow/hardwario_en.svg?style=social&label=Follow)](https://twitter.com/hardwario_en)\n\nThis repository contains Python package [hardwario-chester](https://pypi.org/project/hardwario-chester/)\n\n\n## License\n\nThis project is licensed under the [MIT License](https://opensource.org/licenses/MIT/) - see the [LICENSE](LICENSE) file for details.\n\n---\n\nMade with &#x2764;&nbsp; by [**HARDWARIO a.s.**](https://www.hardwario.com/) in the heart of Europe.\n',
    'author': 'Karel Blavka',
    'author_email': 'karel.blavka@hardwario.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/hardwario/py-hardwario-chester',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4',
}


setup(**setup_kwargs)
